<?php

declare(strict_types=1);

namespace CreoCode\ContaoTourismBundle\EventListener;

use Contao\Input;
use Contao\PageModel;
use Contao\Controller;
use CreoCode\ContaoTourismBundle\Models\AccommodationsModel;
use CreoCode\ContaoTourismBundle\Models\OffersModel;

class ChangelanguageNavigationListener
{


    public function onChangelanguageNavigation(\Terminal42\ChangeLanguage\Event\ChangelanguageNavigationEvent $event) {

		/** @var PageModel $objPage */
		global $objPage;

        // if $objPage->accommodation or $objPage->offer is not set, return
        if(!$objPage->accommodation && !$objPage->offer) {
            return;
        } else if ($objPage->accommodation) {
            // hier AccommodationsModel definieren
            $model = 'AccommodationsModel';

        } else if ($objPage->offer) {
            // hier OffersModel definieren
            $model = 'OffersModel';
        }

        $navigationItem = $event->getNavigationItem();

        if ($navigationItem->isCurrentPage()) {
            if($model == 'AccommodationsModel') {
                $event->getUrlParameterBag()->setUrlAttribute($objPage->accommodation['alias'], false);
                return;
            } else if($model == 'OffersModel') {
                $event->getUrlParameterBag()->setUrlAttribute($objPage->offer['alias'], false);
                return;
            }
        }

        if($model == 'AccommodationsModel') {
            $current = $model::findByIdOrAlias($objPage->accommodation['id']);
        } else if($model == 'OffersModel') {
            $current = $model::findByIdOrAlias($objPage->offer['id']);
        }        

        $t = $model::getTable();
        $parent = $current->getRelated('pid');

        if (0 === (int) $parent->master) {
            $mainId = (int) $current->id;
            $masterId = (int) $current->pid;
        } else {
            $mainId = (int) $current->languageMain;
            $masterId = (int) $parent->master;
        }

        // Abort if current record has no translated version
        if (0 === $mainId || 0 === $masterId) {
            $navigationItem->setIsDirectFallback(false);
            return;
        }

        $targetPage = $navigationItem->getTargetPage();

        if (null === $targetPage) {
            return;
        }

        $translated = $model::findOneBy(
            [
                "($t.id=? OR $t.languageMain=?)",
                sprintf('%s.pid=(SELECT id FROM %s WHERE (id=? OR master=?) AND jumpTo=?)', $t, $parent::getTable()),
            ],
            [$mainId, $mainId, $masterId, $masterId, $targetPage->id]
        );

        if (null === $translated) {
            $navigationItem->setIsDirectFallback(false);
            return;
        }

        // Set the alias of the translated page         
        $event->getUrlParameterBag()->setUrlAttribute($translated->alias ?: $translated->id, false);
    }
}