<?php

namespace CreoCode\ContaoTourismBundle\Models;

use Contao\Model\Collection;

use Contao\Model;
use Contao\Date;

/**
 * Reads and writes prices
 *
 * @property integer $id
 * @property integer $pid
 * @property integer $tstamp
 * @property string  $headline
 * @property string  $alias
 * @property integer $date
 * @property integer $time
 * @property string  $cssClass
 * @property boolean $featured
 * @property boolean $published
 * @property string  $start
 * @property string  $stop
 *
 * @method static PricesModel|null findById($id, array $opt=array())
 * @method static PricesModel|null findByPk($id, array $opt=array())
 * @method static PricesModel|null findByIdOrAlias($val, array $opt=array())
 * @method static PricesModel|null findOneBy($col, $val, array $opt=array())
 * @method static PricesModel|null findOneByPid($val, array $opt=array())
 * @method static PricesModel|null findOneByTstamp($val, array $opt=array())
 * @method static PricesModel|null findOneByHeadline($val, array $opt=array())
 * @method static PricesModel|null findOneByAlias($val, array $opt=array())
 * @method static PricesModel|null findOneByDate($val, array $opt=array())
 * @method static PricesModel|null findOneByTime($val, array $opt=array())
 * @method static PricesModel|null findOneByCssClass($val, array $opt=array())
 * @method static PricesModel|null findOneByFeatured($val, array $opt=array())
 * @method static PricesModel|null findOneByPublished($val, array $opt=array())
 * @method static PricesModel|null findOneByStart($val, array $opt=array())
 * @method static PricesModel|null findOneByStop($val, array $opt=array())
 *
 * @method static Collection|PricesModel[]|PricesModel|null findByPid($val, array $opt=array())
 * @method static Collection|PricesModel[]|PricesModel|null findByTstamp($val, array $opt=array())
 * @method static Collection|PricesModel[]|PricesModel|null findByHeadline($val, array $opt=array())
 * @method static Collection|PricesModel[]|PricesModel|null findByAlias($val, array $opt=array())
 * @method static Collection|PricesModel[]|PricesModel|null findByDate($val, array $opt=array())
 * @method static Collection|PricesModel[]|PricesModel|null findByTime($val, array $opt=array())
 * @method static Collection|PricesModel[]|PricesModel|null findByCssClass($val, array $opt=array())
 * @method static Collection|PricesModel[]|PricesModel|null findByFeatured($val, array $opt=array())
 * @method static Collection|PricesModel[]|PricesModel|null findByPublished($val, array $opt=array())
 * @method static Collection|PricesModel[]|PricesModel|null findByStart($val, array $opt=array())
 * @method static Collection|PricesModel[]|PricesModel|null findByStop($val, array $opt=array())
 * @method static Collection|PricesModel[]|PricesModel|null findMultipleByIds($val, array $opt=array())
 * @method static Collection|PricesModel[]|PricesModel|null findBy($col, $val, array $opt=array())
 * @method static Collection|PricesModel[]|PricesModel|null findAll(array $opt=array())
 *
 * @method static integer countById($id, array $opt=array())
 * @method static integer countByPid($val, array $opt=array())
 * @method static integer countByTstamp($val, array $opt=array())
 * @method static integer countByHeadline($val, array $opt=array())
 * @method static integer countByAlias($val, array $opt=array())
 * @method static integer countByDate($val, array $opt=array())
 * @method static integer countByTime($val, array $opt=array())
 * @method static integer countByCssClass($val, array $opt=array())
 * @method static integer countByFeatured($val, array $opt=array())
 * @method static integer countByPublished($val, array $opt=array())
 * @method static integer countByStart($val, array $opt=array())
 * @method static integer countByStop($val, array $opt=array())
 */
class PricesModel extends Model
{

	/**
	 * Table name
	 * @var string
	 */
	protected static $strTable = 'tl_prices';

	/**
	 * Find a published prices item from one or more prices archives by its ID or alias
	 *
	 * @param mixed $varId      The numeric ID or alias name
	 * @param array $arrPids    An array of parent IDs
	 * @param array $arrOptions An optional options array
	 *
	 * @return PricesModel|null The model or null if there are no prices
	 */
	public static function findPublishedByParentAndIdOrAlias($varId, $arrPids, array $arrOptions=array())
	{
		if (empty($arrPids) || !\is_array($arrPids))
		{
			return null;
		}

		$t = static::$strTable;
		$arrColumns = !preg_match('/^[1-9]\d*$/', $varId) ? array("BINARY $t.alias=?") : array("$t.id=?");
		$arrColumns[] = "$t.pid IN(" . implode(',', array_map('\intval', $arrPids)) . ")";

		if (!static::isPreviewMode($arrOptions))
		{
			$time = Date::floorToMinute();
			$arrColumns[] = "$t.published=1 AND ($t.start='' OR $t.start<='$time') AND ($t.stop='' OR $t.stop>'$time')";
		}

		return static::findOneBy($arrColumns, $varId, $arrOptions);
	}

	/**
	 * Find published prices items by their parent ID
	 *
	 * @param array   $arrPids     An array of prices archive IDs
	 * @param boolean $blnFeatured If true, return only featured prices, if false, return only unfeatured prices
	 * @param integer $intLimit    An optional limit
	 * @param integer $intOffset   An optional offset
	 * @param array   $arrOptions  An optional options array
	 *
	 * @return Collection|PricesModel[]|PricesModel|null A collection of models or null if there are no prices
	 */
	public static function findPublishedByPids($arrPids, $blnFeatured=null, $intLimit=0, $intOffset=0, array $arrOptions=array())
	{
		if (empty($arrPids) || !\is_array($arrPids))
		{
			return null;
		}

		$t = static::$strTable;
		$arrColumns = array("$t.pid IN(" . implode(',', array_map('\intval', $arrPids)) . ")");

		if ($blnFeatured === true)
		{
			$arrColumns[] = "$t.featured='1'";
		}
		elseif ($blnFeatured === false)
		{
			$arrColumns[] = "$t.featured=''";
		}

		// Never return unpublished elements in the back end, so they don't end up in the RSS feed
		if (!static::isPreviewMode($arrOptions))
		{
			$time = Date::floorToMinute();
			$arrColumns[] = "($t.start='' OR $t.start<='$time') AND ($t.stop='' OR $t.stop>'" . ($time + 60) . "') AND $t.published='1'";
		}

		if (!isset($arrOptions['order']))
		{
			$arrOptions['order']  = "$t.date DESC";
		}

		$arrOptions['limit']  = $intLimit;
		$arrOptions['offset'] = $intOffset;

		return static::findBy($arrColumns, null, $arrOptions);
	}

	/**
	 * Count published prices items by their parent ID
	 *
	 * @param array   $arrPids     An array of prices archive IDs
	 * @param boolean $blnFeatured If true, return only featured prices, if false, return only unfeatured prices
	 * @param array   $arrOptions  An optional options array
	 *
	 * @return integer The number of prices items
	 */
	public static function countPublishedByPids($arrPids, $blnFeatured=null, array $arrOptions=array())
	{
		if (empty($arrPids) || !\is_array($arrPids))
		{
			return 0;
		}

		$t = static::$strTable;
		$arrColumns = array("$t.pid IN(" . implode(',', array_map('\intval', $arrPids)) . ")");

		if ($blnFeatured === true)
		{
			$arrColumns[] = "$t.featured='1'";
		}
		elseif ($blnFeatured === false)
		{
			$arrColumns[] = "$t.featured=''";
		}

		if (!static::isPreviewMode($arrOptions))
		{
			$time = Date::floorToMinute();
			$arrColumns[] = "($t.start='' OR $t.start<='$time') AND ($t.stop='' OR $t.stop>'" . ($time + 60) . "') AND $t.published='1'";
		}

		return static::countBy($arrColumns, null, $arrOptions);
	}

	/**
	 * Find published prices items by their parent ID
	 *
	 * @param integer $intPid     The prices archive ID
	 * @param array   $arrOptions An optional options array
	 *
	 * @return Collection|PricesModel[]|PricesModel|null A collection of models or null if there are no prices
	 */
	public static function findPublishedDefaultByPid($intPid, array $arrOptions=array())
	{
		$t = static::$strTable;
		$arrColumns = array("$t.pid=?");

		if (!static::isPreviewMode($arrOptions))
		{
			$time = Date::floorToMinute();
			$arrColumns[] = "($t.start='' OR $t.start<='$time') AND ($t.stop='' OR $t.stop>'" . ($time + 60) . "') AND $t.published='1'";
		}

		if (!isset($arrOptions['order']))
		{
			$arrOptions['order'] = "$t.date DESC";
		}

		return static::findBy($arrColumns, $intPid, $arrOptions);
	}

	/**
	 * Find published prices items by their parent ID
	 *
	 * @param integer $intId      The prices archive ID
	 * @param integer $intLimit   An optional limit
	 * @param array   $arrOptions An optional options array
	 *
	 * @return Collection|PricesModel[]|PricesModel|null A collection of models or null if there are no prices
	 */
	public static function findPublishedByPid($intId, $intLimit=0, array $arrOptions=array())
	{
		$t = static::$strTable;
		$arrColumns = array("$t.pid=?");

		if (!static::isPreviewMode($arrOptions))
		{
			$time = Date::floorToMinute();
			$arrColumns[] = "($t.start='' OR $t.start<='$time') AND ($t.stop='' OR $t.stop>'" . ($time + 60) . "') AND $t.published='1'";
		}

		if (!isset($arrOptions['order']))
		{
			$arrOptions['order'] = "$t.date DESC";
		}

		if ($intLimit > 0)
		{
			$arrOptions['limit'] = $intLimit;
		}

		return static::findBy($arrColumns, $intId, $arrOptions);
	}

	/**
	 * Find all published prices items of a certain period of time by their parent ID
	 *
	 * @param integer $intFrom    The start date as Unix timestamp
	 * @param integer $intTo      The end date as Unix timestamp
	 * @param array   $arrPids    An array of prices archive IDs
	 * @param integer $intLimit   An optional limit
	 * @param integer $intOffset  An optional offset
	 * @param array   $arrOptions An optional options array
	 *
	 * @return Collection|PricesModel[]|PricesModel|null A collection of models or null if there are no prices
	 */
	public static function findPublishedFromToByPids($intFrom, $intTo, $arrPids, $intLimit=0, $intOffset=0, array $arrOptions=array())
	{
		if (empty($arrPids) || !\is_array($arrPids))
		{
			return null;
		}

		$t = static::$strTable;
		$arrColumns = array("$t.date>=? AND $t.date<=? AND $t.pid IN(" . implode(',', array_map('\intval', $arrPids)) . ")");

		if (!static::isPreviewMode($arrOptions))
		{
			$time = Date::floorToMinute();
			$arrColumns[] = "($t.start='' OR $t.start<='$time') AND ($t.stop='' OR $t.stop>'" . ($time + 60) . "') AND $t.published='1'";
		}

		if (!isset($arrOptions['order']))
		{
			$arrOptions['order']  = "$t.date DESC";
		}

		$arrOptions['limit']  = $intLimit;
		$arrOptions['offset'] = $intOffset;

		return static::findBy($arrColumns, array($intFrom, $intTo), $arrOptions);
	}

	/**
	 * Count all published prices items of a certain period of time by their parent ID
	 *
	 * @param integer $intFrom    The start date as Unix timestamp
	 * @param integer $intTo      The end date as Unix timestamp
	 * @param array   $arrPids    An array of prices archive IDs
	 * @param array   $arrOptions An optional options array
	 *
	 * @return integer The number of prices items
	 */
	public static function countPublishedFromToByPids($intFrom, $intTo, $arrPids, array $arrOptions=array())
	{
		if (empty($arrPids) || !\is_array($arrPids))
		{
			return null;
		}

		$t = static::$strTable;
		$arrColumns = array("$t.date>=? AND $t.date<=? AND $t.pid IN(" . implode(',', array_map('\intval', $arrPids)) . ")");

		if (!static::isPreviewMode($arrOptions))
		{
			$time = Date::floorToMinute();
			$arrColumns[] = "($t.start='' OR $t.start<='$time') AND ($t.stop='' OR $t.stop>'" . ($time + 60) . "') AND $t.published='1'";
		}

		return static::countBy($arrColumns, array($intFrom, $intTo), $arrOptions);
	}
}

class_alias(PricesModel::class, 'PricesModel');
