<?php

namespace CreoCode\ContaoTourismBundle\Models;

use Contao\Model\Collection;

use Contao\Model;
use Contao\Date;

/**
 * Reads and writes prices tables
 *
 * @property integer $id
 * @property integer $tstamp
 * @property string  $title
 * @property integer $jumpTo
 * @property boolean $protected
 * @property string  $groups
 *
 * @method static PricesTableModel|null findById($id, array $opt=array())
 * @method static PricesTableModel|null findByPk($id, array $opt=array())
 * @method static PricesTableModel|null findByIdOrAlias($val, array $opt=array())
 * @method static PricesTableModel|null findOneBy($col, $val, array $opt=array())
 * @method static PricesTableModel|null findOneByTstamp($val, array $opt=array())
 * @method static PricesTableModel|null findOneByTitle($val, array $opt=array())
 * @method static PricesTableModel|null findOneByJumpTo($val, array $opt=array())
 * @method static PricesTableModel|null findOneByProtected($val, array $opt=array())
 * @method static PricesTableModel|null findOneByGroups($val, array $opt=array())
 *
 * @method static Collection|PricesTableModel[]|PricesTableModel|null findByTstamp($val, array $opt=array())
 * @method static Collection|PricesTableModel[]|PricesTableModel|null findByTitle($val, array $opt=array())
 * @method static Collection|PricesTableModel[]|PricesTableModel|null findByJumpTo($val, array $opt=array())
 * @method static Collection|PricesTableModel[]|PricesTableModel|null findByProtected($val, array $opt=array())
 * @method static Collection|PricesTableModel[]|PricesTableModel|null findByGroups($val, array $opt=array())
 * @method static Collection|PricesTableModel[]|PricesTableModel|null findMultipleByIds($val, array $opt=array())
 * @method static Collection|PricesTableModel[]|PricesTableModel|null findBy($col, $val, array $opt=array())
 * @method static Collection|PricesTableModel[]|PricesTableModel|null findAll(array $opt=array())
 *
 * @method static integer countById($id, array $opt=array())
 * @method static integer countByTstamp($val, array $opt=array())
 * @method static integer countByTitle($val, array $opt=array())
 * @method static integer countByJumpTo($val, array $opt=array())
 * @method static integer countByProtected($val, array $opt=array())
 * @method static integer countByGroups($val, array $opt=array())
 *
 */
class PricesTableModel extends Model
{

	/**
	 * Table name
	 * @var string
	 */
	protected static $strTable = 'tl_prices_table';

    /**
     * Find published prices items by their parent ID
     *
     * @param array   $arrPids     An array of prices archive IDs
     * @param boolean $blnFeatured If true, return only featured prices, if false, return only unfeatured prices
     * @param integer $intLimit    An optional limit
     * @param integer $intOffset   An optional offset
     * @param array   $arrOptions  An optional options array
     *
     * @return Collection|PricesModel[]|PricesModel|null A collection of models or null if there are no prices
     */
    public static function findPublishedByPids($arrPids, $blnFeatured=null, $intLimit=0, $intOffset=0, array $arrOptions=array())
    {

        if (empty($arrPids) || !\is_array($arrPids))
        {
            return null;
        }

        $t = static::$strTable;
        $arrColumns = array("$t.pid IN(" . implode(',', array_map('\intval', $arrPids)) . ")");

        if ($blnFeatured === true)
        {
            $arrColumns[] = "$t.featured='1'";
        }
        elseif ($blnFeatured === false)
        {
            $arrColumns[] = "$t.featured=''";
        }

        // Never return unpublished elements in the back end, so they don't end up in the RSS feed
		if (!static::isPreviewMode($arrOptions))
		{
            $time = Date::floorToMinute();
            $arrColumns[] = "($t.start='' OR $t.start<='$time') AND ($t.stop='' OR $t.stop>'" . ($time + 60) . "') AND $t.published='1'";
        }

        if (!isset($arrOptions['order']))
        {
            $arrOptions['order']  = "$t.date DESC";
        }

        $arrOptions['limit']  = $intLimit;
        $arrOptions['offset'] = $intOffset;


        return static::findBy($arrColumns, null, $arrOptions);
    }

    /**
     * Find published prices items by their parent ID
     *
     * @param array   $arrPids     An array of prices archive IDs
     * @param boolean $blnFeatured If true, return only featured prices, if false, return only unfeatured prices
     * @param integer $intLimit    An optional limit
     * @param integer $intOffset   An optional offset
     * @param array   $arrOptions  An optional options array
     *
     * @return Collection|PricesModel[]|PricesModel|null A collection of models or null if there are no prices
     */
    public static function findPublishedByPidsAndKeys($arrPids, $blnFeatured=null, $intLimit=0, $intOffset=0, array $arrOptions=array())
    {

        if (empty($arrPids) || !\is_array($arrPids))
        {
            return null;
        }

        $t = static::$strTable;
        $arrColumns = array("$t.pid IN(" . implode(',', array_map('\intval', $arrPids)) . ")");

        if (isset($arrOptions['keys']))
        {
            $keysSorted = [];
            foreach ($arrOptions['keys'] as $value) {
                $chunks = explode('__', $value);
                $keysSorted[] = $chunks[1];
            }

            $arrColumns[] = "$t.pricesFor IN('" . implode("','", $keysSorted) . "')";
        }

        if ($blnFeatured === true)
        {
            $arrColumns[] = "$t.featured='1'";
        }
        elseif ($blnFeatured === false)
        {
            $arrColumns[] = "$t.featured=''";
        }

        // Never return unpublished elements in the back end, so they don't end up in the RSS feed
		if (!static::isPreviewMode($arrOptions))
		{
            $time = Date::floorToMinute();
            $arrColumns[] = "($t.start='' OR $t.start<='$time') AND ($t.stop='' OR $t.stop>'" . ($time + 60) . "') AND $t.published='1'";
        }

        if (!isset($arrOptions['order']))
        {
            $arrOptions['order']  = "$t.date DESC";
        }

        $arrOptions['limit']  = $intLimit;
        $arrOptions['offset'] = $intOffset;



        return static::findBy($arrColumns, null, $arrOptions);
    }
}

class_alias(PricesTableModel::class, 'PricesTableModel');
