<?php

namespace CreoCode\ContaoTourismBundle\Modules;

use Contao\CoreBundle\Exception\InternalServerErrorException;
use Contao\CoreBundle\Exception\PageNotFoundException;
use Contao\CoreBundle\Exception\RedirectResponseException;
use Contao\CoreBundle\Routing\ResponseContext\HtmlHeadBag\HtmlHeadBag;

use Contao\Config;
use Contao\Input;
use Contao\StringUtil;
use Contao\System;
use Contao\Environment;
use Contao\BackendTemplate;

use CreoCode\ContaoTourismBundle\Models\AccommodationsModel;

/**
 * Front end module "accommodations reader".
 *
 * @property array    $accommodations_sets
 */
class ModuleAccommodationsReader extends ModuleAccommodations
{

	/**
	 * Template
	 * @var string
	 */
	protected $strTemplate = 'mod_accommodationsreader';

	/**
	 * Display a wildcard in the back end
	 *
	 * @throws InternalServerErrorException
	 *
	 * @return string
	 */
	public function generate()
	{
		$request = System::getContainer()->get('request_stack')->getCurrentRequest();

		if ($request && System::getContainer()->get('contao.routing.scope_matcher')->isBackendRequest($request))
		{
			$objTemplate = new BackendTemplate('be_wildcard');
			$objTemplate->wildcard = '### ' . $GLOBALS['TL_LANG']['FMD']['accommodationsreader'][0] . ' ###';
			$objTemplate->title = $this->headline;
			$objTemplate->id = $this->id;
			$objTemplate->link = $this->name;
			$objTemplate->href = 'contao/main.php?do=themes&amp;table=tl_module&amp;act=edit&amp;id=' . $this->id;

			return $objTemplate->parse();
		}

		// Return an empty string if "auto_item" is not set to combine list and reader on same page
		if (Input::get('auto_item') === null)
		{
			return '';
		}

		// Return an empty string if "items" is not set (to combine list and reader on same page)
		//if (!Input::get('items'))
		//{
		//	return '';
		//}

		$this->accommodations_sets = $this->sortOutProtected(StringUtil::deserialize($this->accommodations_sets));

		if (empty($this->accommodations_sets) || !\is_array($this->accommodations_sets))
		{
			throw new InternalServerErrorException('The accommodations reader ID ' . $this->id . ' has no sets specified.', $this->id);
		}

		return parent::generate();
	}

	/**
	 * Generate the module
	 */
	protected function compile()
	{
		/** @var PageModel $objPage */
		global $objPage;

		$this->Template->articles = '';
		$this->Template->referer = 'javascript:history.go(-1)';
		$this->Template->back = $GLOBALS['TL_LANG']['MSC']['goBack'];

		// Get the accommodations item
		$objArticle = AccommodationsModel::findPublishedByParentAndIdOrAlias(Input::get('auto_item'), $this->accommodations_sets);

		// Set object variables for language support
		$objPage->accommodation = [
			'reader' => true,
			'alias' => $objArticle->alias,
			'id' => $objArticle->id,
			'pid' => $objArticle->pid,
		];

		// The accommodations item does not exist or has an external target (see #33)
		if (null === $objArticle || $objArticle->source != 'default')
		{
			throw new PageNotFoundException('Page not found: ' . Environment::get('uri'));
		}

		$arrArticle = $this->parseArticle($objArticle);
		$this->Template->articles = $arrArticle;

		// Overwrite the page metadata (see #2853, #4955 and #87)
		$responseContext = System::getContainer()->get('contao.routing.response_context_accessor')->getResponseContext();

		if ($responseContext && $responseContext->has(HtmlHeadBag::class))
		{
			/** @var HtmlHeadBag $htmlHeadBag */
			$htmlHeadBag = $responseContext->get(HtmlHeadBag::class);
			$htmlDecoder = System::getContainer()->get('contao.string.html_decoder');

			if ($objArticle->pageTitle)
			{
				$htmlHeadBag->setTitle($objArticle->pageTitle);
			}
			elseif ($objArticle->headline)
			{
				$htmlHeadBag->setTitle($htmlDecoder->inputEncodedToPlainText($objArticle->headline));
			}

			// Overwrite the page description
			if ($objArticle->description)
			{
				$htmlHeadBag->setMetaDescription($htmlDecoder->inputEncodedToPlainText($objArticle->description));
			}
			elseif ($objArticle->teaser)
			{
				$htmlHeadBag->setMetaDescription($htmlDecoder->htmlToPlainText($objArticle->teaser));
			}
		}
	}
}

class_alias(ModuleAccommodationsReader::class, 'ModuleAccommodationsReader');
