<?php

namespace CreoCode\ContaoTourismBundle\Modules;

use Contao\CoreBundle\Exception\PageNotFoundException;
use Patchwork\Utf8;

use Contao\Environment;
use Contao\System;

/**
 * Front end module "offers set".
 *
 * @property array  $offers_sets
 * @property string $offers_jumpToCurrent
 * @property string $offers_format
 * @property string $offers_order
 * @property int    $offers_readerModule
 */
class ModuleOffersSet extends ModuleOffers
{

	/**
	 * Template
	 * @var string
	 */
	protected $strTemplate = 'mod_offersset';

	/**
	 * Display a wildcard in the back end
	 *
	 * @return string
	 */
	public function generate()
	{
		$request = System::getContainer()->get('request_stack')->getCurrentRequest();

		if ($request && System::getContainer()->get('contao.routing.scope_matcher')->isBackendRequest($request))
		{
			$objTemplate = new BackendTemplate('be_wildcard');
			$objTemplate->wildcard = '### ' . $GLOBALS['TL_LANG']['FMD']['offersset'][0] . ' ###';
			$objTemplate->title = $this->headline;
			$objTemplate->id = $this->id;
			$objTemplate->link = $this->name;
			$objTemplate->href = 'contao/main.php?do=themes&amp;table=tl_module&amp;act=edit&amp;id=' . $this->id;

			return $objTemplate->parse();
		}

		$this->offers_sets = $this->sortOutProtected(StringUtil::deserialize($this->offers_sets));

		// No offers sets available
		if (empty($this->offers_sets) || !\is_array($this->offers_sets))
		{
			return '';
		}

		// Show the offers reader if an item has been selected
		if ($this->offers_readerModule > 0 && (isset($_GET['items']) || (Config::get('useAutoItem') && isset($_GET['auto_item']))))
		{
			return $this->getFrontendModule($this->offers_readerModule, $this->strColumn);
		}

		// Hide the module if no period has been selected
		if ($this->offers_jumpToCurrent == 'hide_module' && !isset($_GET['year']) && !isset($_GET['month']) && !isset($_GET['day']))
		{
			return '';
		}

		return parent::generate();
	}

	/**
	 * Generate the module
	 */
	protected function compile()
	{
		/** @var PageModel $objPage */
		global $objPage;

		$limit = null;
		$offset = 0;
		$intBegin = 0;
		$intEnd = 0;

		$intYear = Input::get('year');
		$intMonth = Input::get('month');
		$intDay = Input::get('day');

		// Jump to the current period
		if (!isset($_GET['year']) && !isset($_GET['month']) && !isset($_GET['day']) && $this->offers_jumpToCurrent != 'all_items')
		{
			switch ($this->offers_format)
			{
				case 'offers_year':
					$intYear = date('Y');
					break;

				default:
				case 'offers_month':
					$intMonth = date('Ym');
					break;

				case 'offers_day':
					$intDay = date('Ymd');
					break;
			}
		}

		// Create the date object
		try
		{
			if ($intYear)
			{
				$strDate = $intYear;
				$objDate = new Date($strDate, 'Y');
				$intBegin = $objDate->yearBegin;
				$intEnd = $objDate->yearEnd;
				$this->headline .= ' ' . date('Y', $objDate->tstamp);
			}
			elseif ($intMonth)
			{
				$strDate = $intMonth;
				$objDate = new Date($strDate, 'Ym');
				$intBegin = $objDate->monthBegin;
				$intEnd = $objDate->monthEnd;
				$this->headline .= ' ' . Date::parse('F Y', $objDate->tstamp);
			}
			elseif ($intDay)
			{
				$strDate = $intDay;
				$objDate = new Date($strDate, 'Ymd');
				$intBegin = $objDate->dayBegin;
				$intEnd = $objDate->dayEnd;
				$this->headline .= ' ' . Date::parse($objPage->dateFormat, $objDate->tstamp);
			}
			elseif ($this->offers_jumpToCurrent == 'all_items')
			{
				$intBegin = 0; // 1970-01-01 00:00:00
				$intEnd = 2145913200; // 2038-01-01 00:00:00
			}
		}
		catch (\OutOfBoundsException $e)
		{
			throw new PageNotFoundException('Page not found: ' . Environment::get('uri'));
		}

		$this->Template->articles = array();

		// Split the result
		if ($this->perPage > 0)
		{
			// Get the total number of items
			$intTotal = OffersModel::countPublishedFromToByPids($intBegin, $intEnd, $this->offers_sets);

			if ($intTotal > 0)
			{
				$total = $intTotal;

				// Get the current page
				$id = 'page_a' . $this->id;
				$page = Input::get($id) ?? 1;

				// Do not index or cache the page if the page number is outside the range
				if ($page < 1 || $page > max(ceil($total/$this->perPage), 1))
				{
					throw new PageNotFoundException('Page not found: ' . Environment::get('uri'));
				}

				// Set limit and offset
				$limit = $this->perPage;
				$offset = (max($page, 1) - 1) * $this->perPage;

				// Add the pagination menu
				$objPagination = new Pagination($total, $this->perPage, Config::get('maxPaginationLinks'), $id);
				$this->Template->pagination = $objPagination->generate("\n  ");
			}
		}

		// Determine sorting
		$t = OffersModel::getTable();
		$arrOptions = array();

		switch ($this->offers_order)
		{
			case 'order_date_asc':
				$arrOptions['order'] = "$t.startDate";
				break;

			case 'order_date_desc':
				$arrOptions['order'] = "$t.startDate DESC";
				break;
				
			case 'order_headline_asc':
				$arrOptions['order'] = "$t.headline";
				break;

			case 'order_headline_desc':
				$arrOptions['order'] = "$t.headline DESC";
				break;

			case 'order_random':
				$arrOptions['order'] = "RAND()";
				break;

			default:
				$arrOptions['order'] = "$t.startDate";
		}

		// Get the offers items
		if (isset($limit))
		{
			$objArticles = OffersModel::findPublishedFromToByPids($intBegin, $intEnd, $this->offers_sets, $limit, $offset, $arrOptions);
		}
		else
		{
			$objArticles = OffersModel::findPublishedFromToByPids($intBegin, $intEnd, $this->offers_sets, 0, 0, $arrOptions);
		}

		// Add the articles
		if ($objArticles !== null)
		{
			$this->Template->articles = $this->parseArticles($objArticles);
		}

		$this->Template->headline = trim($this->headline);
		$this->Template->back = $GLOBALS['TL_LANG']['MSC']['goBack'];
		$this->Template->empty = $GLOBALS['TL_LANG']['MSC']['empty'];
	}
}

class_alias(ModuleOffersSet::class, 'ModuleOffersSet');
