<?php

declare(strict_types=1);

namespace CreoCode\ContaoTourismBundle\Picker;

use Contao\CoreBundle\Framework\FrameworkAwareInterface;
use Contao\CoreBundle\Framework\FrameworkAwareTrait;
use Contao\CoreBundle\Picker\AbstractInsertTagPickerProvider;
use Contao\CoreBundle\Picker\DcaPickerProviderInterface;
use Contao\CoreBundle\Picker\PickerConfig;

use CreoCode\ContaoTourismBundle\Models\AccommodationsSetModel;
use CreoCode\ContaoTourismBundle\Models\AccommodationsModel;

use Knp\Menu\FactoryInterface;
use Symfony\Component\Routing\RouterInterface;
use Symfony\Component\Security\Core\Security;
use Symfony\Contracts\Translation\TranslatorInterface;

class AccommodationsPickerProvider extends AbstractInsertTagPickerProvider implements DcaPickerProviderInterface, FrameworkAwareInterface
{
    use FrameworkAwareTrait;

    /**
     * @internal
     */
    public function __construct(FactoryInterface $menuFactory, RouterInterface $router, TranslatorInterface|null $translator, private Security $security)
    {
        parent::__construct($menuFactory, $router, $translator);
    }

    public function getName(): string
    {
        return 'accommodationsPicker';
    }

    public function supportsContext($context): bool
    {
        return 'link' === $context && $this->security->isGranted('contao_user.modules', 'accommodations');
    }

    public function supportsValue(PickerConfig $config): bool
    {
        return $this->isMatchingInsertTag($config);
    }

    public function getDcaTable(PickerConfig $config = null): string
    {
        return 'tl_accommodations';
    }

    public function getDcaAttributes(PickerConfig $config): array
    {
        $attributes = ['fieldType' => 'radio'];

        if ($this->supportsValue($config)) {
            $attributes['value'] = $this->getInsertTagValue($config);

            if ($flags = $this->getInsertTagFlags($config)) {
                $attributes['flags'] = $flags;
            }
        }

        return $attributes;
    }

    public function convertDcaValue(PickerConfig $config, $value): string
    {
        return sprintf($this->getInsertTag($config), $value);
    }

    protected function getRouteParameters(PickerConfig $config = null): array
    {
        $params = ['do' => 'accommodations'];

        if (null === $config || !$config->getValue() || !$this->supportsValue($config)) {
            return $params;
        }

        if (null !== ($accommodationsSetId = $this->getAccommodationsSetId($this->getInsertTagValue($config)))) {
            $params['table'] = 'tl_accommodations';
            $params['id'] = $accommodationsSetId;
        }

        return $params;
    }

    protected function getDefaultInsertTag(): string
    {
        return '{{accommodations_url::%s}}';
    }

    private function getAccommodationsSetId(int|string $id): int|null
    {
        $accommodationsAdapter = $this->framework->getAdapter(AccommodationsModel::class);

        if (!($accommodationsModel = $accommodationsAdapter->findById($id)) instanceof AccommodationsModel) {
            return null;
        }

        if (!($accommodationsSet = $accommodationsModel->getRelated('pid')) instanceof AccommodationsSetModel) {
            return null;
        }

        return (int) $accommodationsSet->id;
    }
}