<?php

declare(strict_types=1);

namespace CreoCode\ContaoTourismBundle\Picker;

use Contao\CoreBundle\Framework\FrameworkAwareInterface;
use Contao\CoreBundle\Framework\FrameworkAwareTrait;
use Contao\CoreBundle\Picker\AbstractInsertTagPickerProvider;
use Contao\CoreBundle\Picker\DcaPickerProviderInterface;
use Contao\CoreBundle\Picker\PickerConfig;

use CreoCode\ContaoTourismBundle\Models\OffersSetModel;
use CreoCode\ContaoTourismBundle\Models\OffersModel;

use Knp\Menu\FactoryInterface;
use Symfony\Component\Routing\RouterInterface;
use Symfony\Component\Security\Core\Security;
use Symfony\Contracts\Translation\TranslatorInterface;

class OffersPickerProvider extends AbstractInsertTagPickerProvider implements DcaPickerProviderInterface, FrameworkAwareInterface
{
    use FrameworkAwareTrait;

    /**
     * @internal
     */
    public function __construct(FactoryInterface $menuFactory, RouterInterface $router, TranslatorInterface|null $translator, private Security $security)
    {
        parent::__construct($menuFactory, $router, $translator);
    }

    public function getName(): string
    {
        return 'offersPicker';
    }

    public function supportsContext($context): bool
    {
        return 'link' === $context && $this->security->isGranted('contao_user.modules', 'offers');
    }

    public function supportsValue(PickerConfig $config): bool
    {
        return $this->isMatchingInsertTag($config);
    }

    public function getDcaTable(PickerConfig $config = null): string
    {
        return 'tl_offers';
    }

    public function getDcaAttributes(PickerConfig $config): array
    {
        $attributes = ['fieldType' => 'radio'];

        if ($this->supportsValue($config)) {
            $attributes['value'] = $this->getInsertTagValue($config);

            if ($flags = $this->getInsertTagFlags($config)) {
                $attributes['flags'] = $flags;
            }
        }

        return $attributes;
    }

    public function convertDcaValue(PickerConfig $config, $value): string
    {
        return sprintf($this->getInsertTag($config), $value);
    }

    protected function getRouteParameters(PickerConfig $config = null): array
    {
        $params = ['do' => 'offers'];

        if (null === $config || !$config->getValue() || !$this->supportsValue($config)) {
            return $params;
        }

        if (null !== ($offersSetId = $this->getOffersSetId($this->getInsertTagValue($config)))) {
            $params['table'] = 'tl_offers';
            $params['id'] = $offersSetId;
        }

        return $params;
    }

    protected function getDefaultInsertTag(): string
    {
        return '{{offers_url::%s}}';
    }

    private function getOffersSetId(int|string $id): int|null
    {
        $offersAdapter = $this->framework->getAdapter(OffersModel::class);

        if (!($offersModel = $offersAdapter->findById($id)) instanceof OffersModel) {
            return null;
        }

        if (!($offersSet = $offersModel->getRelated('pid')) instanceof OffersSetModel) {
            return null;
        }

        return (int) $offersSet->id;
    }
}