<?php

use Contao\System;
use Symfony\Component\HttpFoundation\Request;

use CreoCode\ContaoTourismBundle\Modules\ModuleAccommodationsList;
use CreoCode\ContaoTourismBundle\Modules\ModuleAccommodationsReader;
use CreoCode\ContaoTourismBundle\Modules\ModuleOffersList;
use CreoCode\ContaoTourismBundle\Modules\ModuleOffersReader;
use CreoCode\ContaoTourismBundle\Modules\ModulePricesList;

use CreoCode\ContaoTourismBundle\Forms\FormCheckBoxAccommodations;
use CreoCode\ContaoTourismBundle\Forms\FormCheckBoxOffers;

use CreoCode\ContaoTourismBundle\Models\AccommodationsModel;
use CreoCode\ContaoTourismBundle\Models\AccommodationsSetModel;

use CreoCode\ContaoTourismBundle\Models\OffersModel;
use CreoCode\ContaoTourismBundle\Models\OffersSetModel;

use CreoCode\ContaoTourismBundle\Models\PricesTableModel;
use CreoCode\ContaoTourismBundle\Models\PricesModel;
use CreoCode\ContaoTourismBundle\Models\PricesSetModel;

/**
 * Back end modules
 */
$GLOBALS['BE_MOD']['tourism'] = [
    'accommodations' => [
        'tables' => ['tl_accommodations_set', 'tl_accommodations', 'tl_content']
    ],
    'offers' => [
        'tables' => ['tl_offers_set', 'tl_offers', 'tl_content']
    ],
    'prices' => [
        'tables' => ['tl_prices_set', 'tl_prices', 'tl_prices_table']
    ],
];

// Move 'tourism' after 'content'
if(isset($GLOBALS['BE_MOD']['content'])) {
    $tourism = $GLOBALS['BE_MOD']['tourism'];
    unset($GLOBALS['BE_MOD']['tourism']);
    $newArray = [];
    foreach ($GLOBALS['BE_MOD'] as $key => $value) {
        $newArray[$key] = $value;
        if ($key === 'content') {
            $newArray['tourism'] = $tourism;
        }
    }
    $GLOBALS['BE_MOD'] = $newArray;
}


/**
 * Front end modules
 */
$GLOBALS['FE_MOD']['tourism'] = [
    'accommodationslist' => ModuleAccommodationsList::class,
    'accommodationsreader' => ModuleAccommodationsReader::class,
    'offerslist' => ModuleOffersList::class,
    'offersreader' => ModuleOffersReader::class,
    'priceslist' => ModulePricesList::class,
];


/**
 * Front end form fields
 */
$GLOBALS['TL_FFL']['checkboxAccommodations'] = FormCheckBoxAccommodations::class;
$GLOBALS['TL_FFL']['checkboxOffers'] = FormCheckBoxOffers::class;


// Stylesheet / if backend
if (System::getContainer()->get('contao.routing.scope_matcher')->isBackendRequest(System::getContainer()->get('request_stack')->getCurrentRequest() ?? Request::create(''))) {
	$GLOBALS['TL_CSS'][] = 'bundles/contaotourism/css/backend.css|static';
}


// Register hooks
$GLOBALS['TL_HOOKS']['getSearchablePages'][] = array('CreoCode\ContaoTourismBundle\Classes\Accommodations', 'getSearchablePages');
$GLOBALS['TL_HOOKS']['getSearchablePages'][] = array('CreoCode\ContaoTourismBundle\Classes\Offers', 'getSearchablePages');

//$GLOBALS['TL_HOOKS']['generatePage'][] = array('contao_tourism.listener.generate_page', 'onGeneratePage');
$GLOBALS['TL_HOOKS']['replaceInsertTags'][] = array('contao_tourism.listener.insert_tags', 'onReplaceInsertTags');

/**
 * Register Models
 */
$GLOBALS['TL_MODELS'][\CreoCode\ContaoTourismBundle\Models\AccommodationsModel::getTable()] = AccommodationsModel::class;
$GLOBALS['TL_MODELS'][\CreoCode\ContaoTourismBundle\Models\AccommodationsSetModel::getTable()] = AccommodationsSetModel::class;

$GLOBALS['TL_MODELS'][\CreoCode\ContaoTourismBundle\Models\OffersModel::getTable()] = OffersModel::class;
$GLOBALS['TL_MODELS'][\CreoCode\ContaoTourismBundle\Models\OffersSetModel::getTable()] = OffersSetModel::class;

$GLOBALS['TL_MODELS'][\CreoCode\ContaoTourismBundle\Models\PricesTableModel::getTable()] = PricesTableModel::class;
$GLOBALS['TL_MODELS'][\CreoCode\ContaoTourismBundle\Models\PricesModel::getTable()] = PricesModel::class;
$GLOBALS['TL_MODELS'][\CreoCode\ContaoTourismBundle\Models\PricesSetModel::getTable()] = PricesSetModel::class;

// Add permissions
$GLOBALS['TL_PERMISSIONS'][] = 'accommodations';
$GLOBALS['TL_PERMISSIONS'][] = 'accommodationsp';
$GLOBALS['TL_PERMISSIONS'][] = 'offers';
$GLOBALS['TL_PERMISSIONS'][] = 'offersp';
$GLOBALS['TL_PERMISSIONS'][] = 'prices';
$GLOBALS['TL_PERMISSIONS'][] = 'pricesp';

/*
 * Dropdown options
 */
Contao\System::loadLanguageFile('tl_tourism');
$GLOBALS['tourism']['currency']['options'] = array(
    'EUR' => $GLOBALS['TL_LANG']['tl_tourism']['EUR'], 
    'USD' => $GLOBALS['TL_LANG']['tl_tourism']['USD'], 
    'JPY' => $GLOBALS['TL_LANG']['tl_tourism']['JPY'], 
    'GBP' => $GLOBALS['TL_LANG']['tl_tourism']['GBP'], 
    'AUD' => $GLOBALS['TL_LANG']['tl_tourism']['AUD'], 
    'CAD' => $GLOBALS['TL_LANG']['tl_tourism']['CAD'], 
    'CHF' => $GLOBALS['TL_LANG']['tl_tourism']['CHF']
);
$GLOBALS['tourism']['currency']['weekdays'] = array(
    'monday' =>     $GLOBALS['TL_LANG']['tl_tourism']['monday'], 
    'tuesday' =>    $GLOBALS['TL_LANG']['tl_tourism']['tuesday'], 
    'wednesday' =>  $GLOBALS['TL_LANG']['tl_tourism']['wednesday'], 
    'thrusday' =>   $GLOBALS['TL_LANG']['tl_tourism']['thrusday'], 
    'friday' =>     $GLOBALS['TL_LANG']['tl_tourism']['friday'], 
    'saturday' =>   $GLOBALS['TL_LANG']['tl_tourism']['saturday'], 
    'sunday' =>     $GLOBALS['TL_LANG']['tl_tourism']['sunday']
);


// Add changelanguageNavigation hook
$GLOBALS['TL_HOOKS']['changelanguageNavigation'][] = [
    'CreoCode\ContaoTourismBundle\EventListener\ChangelanguageNavigationListener',
    'onChangelanguageNavigation'
];