<?php

use Contao\Backend;
use Contao\Input;
use Contao\System;
use Contao\DC_Table;

$GLOBALS['TL_DCA']['tl_accommodations_set'] = array
(

	// Config
	'config' => array
	(
		'dataContainer'               => DC_Table::class,
		'ctable'                      => array('tl_accommodations'),
		'switchToEdit'                => true,
		'enableVersioning'            => true,
		'markAsCopy'                  => 'title',
		//'onload_callback' => array
		//(
		//	array('tl_accommodations_set', 'checkPermission'),
		//),
		//'oncreate_callback' => array
		//(
		//	array('tl_accommodations_set', 'adjustPermissions')
		//),
		//'oncopy_callback' => array
		//(
		//	array('tl_accommodations_set', 'adjustPermissions')
		//),
		//'onsubmit_callback' => array
		//(
		//
		//),
		'sql' => array
		(
			'keys' => array
			(
				'id' => 'primary'
			)
		)
	),

	// List
	'list' => array
	(
		'sorting' => array
		(
			'mode'                    => 1,
			'fields'                  => array('title'),
			'flag'                    => 1,
			'panelLayout'             => 'filter;search,limit'
		),
		'label' => array
		(
			'fields'                  => array('title'),
			'format'                  => '%s'
		),
		'global_operations' => array
		(
			'all' => array
			(
				'label'               => &$GLOBALS['TL_LANG']['MSC']['all'],
				'href'                => 'act=select',
				'class'               => 'header_edit_all',
				'attributes'          => 'onclick="Backend.getScrollOffset()" accesskey="e"'
			)
		),
	),

	// Palettes
	'palettes' => array
	(
		'__selector__'                => array('protected'),
		'default'                     => '{title_legend},title,jumpTo,currency;{protected_legend:hide},protected'
	),

	// Subpalettes
	'subpalettes' => array
	(
		'protected'                   => 'groups'
	),

	// Fields
	'fields' => array
	(
		'id' => array
		(
			'sql'                     => "int(10) unsigned NOT NULL auto_increment"
		),
		'tstamp' => array
		(
			'sql'                     => "int(10) unsigned NOT NULL default 0"
		),
		'title' => array
		(
			'exclude'                 => true,
			'search'                  => true,
			'inputType'               => 'text',
			'eval'                    => array('mandatory'=>true, 'maxlength'=>255, 'tl_class'=>'w50'),
			'sql'                     => "varchar(255) NOT NULL default ''"
		),
		'jumpTo' => array
		(
			'exclude'                 => true,
			'inputType'               => 'pageTree',
			'foreignKey'              => 'tl_page.title',
			'eval'                    => array('mandatory'=>true, 'fieldType'=>'radio', 'tl_class'=>'clr'),
			'sql'                     => "int(10) unsigned NOT NULL default 0",
			'relation'                => array('type'=>'hasOne', 'load'=>'lazy')
		),
        'currency' => array
		(
            'exclude'                 => true,
            'inputType'               => 'select',
            'options'                 => $GLOBALS['tourism']['currency']['options'],
            //'options_callback'        => array('tl_accommodations_set', 'getCurrency'), 
            'eval'                    => array('tl_class' => 'w50'),
            'sql'                     => "varchar(255) NOT NULL default ''"
        ),
		'protected' => array
		(
			'exclude'                 => true,
			'filter'                  => true,
			'inputType'               => 'checkbox',
			'eval'                    => array('submitOnChange'=>true),
			'sql'                     => "char(1) NOT NULL default ''"
		),
		'groups' => array
		(
			'exclude'                 => true,
			'inputType'               => 'checkbox',
			'foreignKey'              => 'tl_member_group.name',
			'eval'                    => array('mandatory'=>true, 'multiple'=>true),
			'sql'                     => "blob NULL",
			'relation'                => array('type'=>'hasMany', 'load'=>'lazy')
		)
	)
);

/**
 * Provide miscellaneous methods that are used by the data configuration array.
 *
 * @property Contao\Accommodations $Accommodations
 */
class tl_accommodations_set extends Backend
{

	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import('Contao\BackendUser', 'User');
	}

	/**
	 * Check permissions to edit table tl_accommodations_set
	 *
	 * @throws Contao\CoreBundle\Exception\AccessDeniedException
	 */
	public function checkPermission()
	{
		$bundles = System::getContainer()->getParameter('kernel.bundles');

		if ($this->User->isAdmin)
		{
			return;
		}

		// Set root IDs
		if (empty($this->User->accommodation) || !\is_array($this->User->accommodation))
		{
			$root = array(0);
		}
		else
		{
			$root = $this->User->accommodation;
		}

		$GLOBALS['TL_DCA']['tl_accommodations_set']['list']['sorting']['root'] = $root;

		// Check permissions to add sets
		if (!$this->User->hasAccess('create', 'accommodationsp'))
		{
			$GLOBALS['TL_DCA']['tl_accommodations_set']['config']['closed'] = true;
			$GLOBALS['TL_DCA']['tl_accommodations_set']['config']['notCreatable'] = true;
			$GLOBALS['TL_DCA']['tl_accommodations_set']['config']['notCopyable'] = true;
		}

		// Check permissions to delete calendars
		if (!$this->User->hasAccess('delete', 'accommodationsp'))
		{
			$GLOBALS['TL_DCA']['tl_accommodations_set']['config']['notDeletable'] = true;
		}

		/** @var Symfony\Component\HttpFoundation\Session\SessionInterface $objSession */
		$objSession = System::getContainer()->get('session');

		// Check current action
		switch (Input::get('act'))
		{
			case 'select':
				// Allow
				break;

			case 'create':
				if (!$this->User->hasAccess('create', 'accommodationsp'))
				{
					throw new Contao\CoreBundle\Exception\AccessDeniedException('Not enough permissions to create accommodation sets.');
				}
				break;

			case 'edit':
			case 'copy':
			case 'delete':
			case 'show':
				if (!\in_array(Input::get('id'), $root) || (Input::get('act') == 'delete' && !$this->User->hasAccess('delete', 'accommodationsp')))
				{
					throw new Contao\CoreBundle\Exception\AccessDeniedException('Not enough permissions to ' . Input::get('act') . ' accommodation set ID ' . Input::get('id') . '.');
				}
				break;

			case 'editAll':
			case 'deleteAll':
			case 'overrideAll':
			case 'copyAll':
				$session = $objSession->all();
				if (Input::get('act') == 'deleteAll' && !$this->User->hasAccess('delete', 'accommodationsp'))
				{
					$session['CURRENT']['IDS'] = array();
				}
				else
				{
					$session['CURRENT']['IDS'] = array_intersect((array) $session['CURRENT']['IDS'], $root);
				}
				$objSession->replace($session);
				break;

			default:
				if (\strlen(Input::get('act')))
				{
					throw new Contao\CoreBundle\Exception\AccessDeniedException('Not enough permissions to ' . Input::get('act') . ' accommodation sets.');
				}
				break;
		}
	}

	/**
	 * Add the new set to the permissions
	 *
	 * @param $insertId
	 */
	public function adjustPermissions($insertId)
	{
		// The oncreate_callback passes $insertId as second argument
		if (\func_num_args() == 4)
		{
			$insertId = func_get_arg(1);
		}

		if ($this->User->isAdmin)
		{
			return;
		}

		// Set root IDs
		if (empty($this->User->accommodation) || !\is_array($this->User->accommodation))
		{
			$root = array(0);
		}
		else
		{
			$root = $this->User->accommodation;
		}

		// The set is enabled already
		if (\in_array($insertId, $root))
		{
			return;
		}

		/** @var Symfony\Component\HttpFoundation\Session\Attribute\AttributeBagInterface $objSessionBag */
		$objSessionBag = System::getContainer()->get('session')->getBag('contao_backend');

		$arrNew = $objSessionBag->get('new_records');

		if (\is_array($arrNew['tl_accommodations_set']) && \in_array($insertId, $arrNew['tl_accommodations_set']))
		{
			// Add the permissions on group level
			if ($this->User->inherit != 'custom')
			{
				$objGroup = $this->Database->execute("SELECT id, accommodation, accommodationsp FROM tl_user_group WHERE id IN(" . implode(',', array_map('\intval', $this->User->groups)) . ")");

				while ($objGroup->next())
				{
					$arrNewp = Contao\StringUtil::deserialize($objGroup->accommodationsp);

					if (\is_array($arrNewp) && \in_array('create', $arrNewp))
					{
						$arrAccommodations = Contao\StringUtil::deserialize($objGroup->accommodation, true);
						$arrAccommodations[] = $insertId;

						$this->Database->prepare("UPDATE tl_user_group SET accommodation=? WHERE id=?")
									   ->execute(serialize($arrAccommodations), $objGroup->id);
					}
				}
			}

			// Add the permissions on user level
			if ($this->User->inherit != 'group')
			{
				$objUser = $this->Database->prepare("SELECT accommodation, accommodationsp FROM tl_user WHERE id=?")
										   ->limit(1)
										   ->execute($this->User->id);

				$arrNewp = Contao\StringUtil::deserialize($objUser->accommodationsp);

				if (\is_array($arrNewp) && \in_array('create', $arrNewp))
				{
					$arrAccommodations = Contao\StringUtil::deserialize($objUser->accommodation, true);
					$arrAccommodations[] = $insertId;

					$this->Database->prepare("UPDATE tl_user SET accommodation=? WHERE id=?")
								   ->execute(serialize($arrAccommodations), $this->User->id);
				}
			}

			// Add the new element to the user object
			$root[] = $insertId;
			$this->User->accommodation = $root;
		}
	}

}
