<?php

use Contao\Backend;
use Contao\BackendUser;

/**
 * Add palettes to tl_form_field
 */
$GLOBALS['TL_DCA']['tl_form_field']['palettes']['checkboxAccommodations'] = '{type_legend},type,name,label;{fconfig_legend},mandatory;{options_legend},accommodations_sets;{expert_legend:hide},class;{template_legend:hide},customTpl;{invisible_legend:hide},invisible';

$GLOBALS['TL_DCA']['tl_form_field']['palettes']['checkboxOffers'] = '{type_legend},type,name,label;{fconfig_legend},mandatory;{options_legend},offers_sets;{expert_legend:hide},class;{template_legend:hide},customTpl;{invisible_legend:hide},invisible';

/**
 * Add fields to tl_form_field
 */
$GLOBALS['TL_DCA']['tl_form_field']['fields']['accommodations_sets'] = array
    (
    'label' => &$GLOBALS['TL_LANG']['tl_form_field']['accommodations_sets'],
	'exclude'                 => true,
	'inputType'               => 'checkbox',
	'options_callback'        => array('tl_form_field_tourism', 'getAccommodationsSet'),
	'eval'                    => array('multiple'=>true, 'mandatory'=>true),
	'sql'                     => "blob NULL"
);
$GLOBALS['TL_DCA']['tl_form_field']['fields']['offers_sets'] = array
    (
    'label'					  => &$GLOBALS['TL_LANG']['tl_form_field']['offers_sets'],
	'exclude'                 => true,
	'inputType'               => 'checkbox',
	'options_callback'        => array('tl_form_field_tourism', 'getOffersSet'),
	'eval'                    => array('multiple'=>true, 'mandatory'=>true),
	'sql'                     => "blob NULL"
);


/**
 * Class tl_form_field
 */
class tl_form_field_tourism extends Backend {

	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import(BackendUser::class, 'User');
	}

	/**
	 * Get all accommodations sets and return them as array
	 *
	 * @return array
	 */
	public function getAccommodationsSet()
	{
		if (!$this->User->isAdmin && !\is_array($this->User->accommodations))
		{
			return array();
		}

		$arrSets = array();
		$objSets = $this->Database->execute("SELECT id, title FROM tl_accommodations_set ORDER BY title");

		while ($objSets->next())
		{
			if ($this->User->hasAccess($objSets->id, 'accommodations'))
			{
				$arrSets[$objSets->id] = $objSets->title;
			}
		}

		return $arrSets;
	}

	/**
	 * Get all offers sets and return them as array
	 *
	 * @return array
	 */
	public function getOffersSet()
	{
		if (!$this->User->isAdmin && !\is_array($this->User->offers))
		{
			return array();
		}

		$arrSets = array();
		$objSets = $this->Database->execute("SELECT id, title FROM tl_offers_set ORDER BY title");

		while ($objSets->next())
		{
			if ($this->User->hasAccess($objSets->id, 'offers'))
			{
				$arrSets[$objSets->id] = $objSets->title;
			}
		}

		return $arrSets;
	}

}