<?php

use Contao\Backend;
use CreoCode\ContaoTourismBundle\Models\PricesSetModel;

// Add palettes to tl_module
$GLOBALS['TL_DCA']['tl_module']['palettes']['accommodationslist'] = '{title_legend},name,headline,type;{config_legend},accommodations_sets,accommodations_readerModule,numberOfItems,accommodations_featured,accommodations_order,skipFirst,perPage;{template_legend:hide},accommodations_template,customTpl;{image_legend:hide},imgSize;{protected_legend:hide},protected;{expert_legend:hide},guests,cssID';
$GLOBALS['TL_DCA']['tl_module']['palettes']['accommodationsreader'] = '{title_legend},name,headline,type;{config_legend},accommodations_sets;{template_legend:hide},accommodations_template,customTpl;{image_legend:hide},imgSize;{protected_legend:hide},protected;{expert_legend:hide},guests,cssID';


// Add fields to tl_module
$GLOBALS['TL_DCA']['tl_module']['fields']['accommodations_sets'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['accommodations_sets'],
	'exclude'                 => true,
	'inputType'               => 'checkbox',
	'options_callback'        => array('tl_module_accommodations', 'getAccommodationsSet'),
	'eval'                    => array('multiple'=>true, 'mandatory'=>true),
	'sql'                     => "blob NULL"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['accommodations_featured'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['accommodations_featured'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options'                 => array('all_items', 'featured', 'unfeatured'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('tl_class'=>'w50 clr'),
	'sql'                     => "varchar(16) NOT NULL default 'all_items'"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['accommodations_jumpToCurrent'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['accommodations_jumpToCurrent'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options'                 => array('hide_module', 'show_current', 'all_items'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('tl_class'=>'w50'),
	'sql'                     => "varchar(16) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['accommodations_readerModule'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['accommodations_readerModule'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options_callback'        => array('tl_module_accommodations', 'getReaderModules'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('includeBlankOption'=>true, 'tl_class'=>'w50'),
	'sql'                     => "int(10) unsigned NOT NULL default 0"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['accommodations_template'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['accommodations_template'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options_callback'        => array('tl_module_accommodations', 'getAccommodationsTemplates'),
	'eval'                    => array('tl_class'=>'w50'),
	'sql'                     => "varchar(64) NOT NULL default 'accommodations_latest'"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['accommodations_format'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['accommodations_format'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options'                 => array('accommodations_day', 'accommodations_month', 'accommodations_year'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('tl_class'=>'w50 clr', 'addWizardClass'=>false),
	'wizard' => array
	(
		array('tl_module_accommodations', 'hideStartDay')
	),
	'sql'                     => "varchar(32) NOT NULL default 'accommodations_month'"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['accommodations_order'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['accommodations_order'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options_callback'        => array('tl_module_accommodations', 'getSortingOptions'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('tl_class'=>'w50'),
	'sql'                     => "varchar(32) NOT NULL default 'order_sorting_asc'"
);


/**
 * Provide miscellaneous methods that are used by the data configuration array.
 */
class tl_module_accommodations extends Backend
{

	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import('Contao\BackendUser', 'User');
	}

	/**
	 * Get all accommodations sets and return them as array
	 *
	 * @return array
	 */
	public function getAccommodationsSet()
	{
		if (!$this->User->isAdmin && !\is_array($this->User->accommodations))
		{
			return array();
		}

		$arrSets = array();
		$objSets = $this->Database->execute("SELECT id, title FROM tl_accommodations_set ORDER BY title");

		while ($objSets->next())
		{
			if ($this->User->hasAccess($objSets->id, 'accommodations'))
			{
				$arrSets[$objSets->id] = $objSets->title;
			}
		}

		return $arrSets;
	}

	/**
	 * Get all accommodations reader modules and return them as array
	 *
	 * @return array
	 */
	public function getReaderModules()
	{
		$arrModules = array();
		$objModules = $this->Database->execute("SELECT m.id, m.name, t.name AS theme FROM tl_module m LEFT JOIN tl_theme t ON m.pid=t.id WHERE m.type='accommodationsreader' ORDER BY t.name, m.name");

		while ($objModules->next())
		{
			$arrModules[$objModules->theme][$objModules->id] = $objModules->name . ' (ID ' . $objModules->id . ')';
		}

		return $arrModules;
	}

	/**
	 * Hide the start day drop-down if not applicable
	 *
	 * @return string
	 */
	public function hideStartDay()
	{
		return '
  <script>
    var enableStartDay = function() {
      var e1 = $("ctrl_accommodations_startDay").getParent("div");
      var e2 = $("ctrl_accommodations_order").getParent("div");
      if ($("ctrl_accommodations_format").value == "accommodations_day") {
        e1.setStyle("display", "block");
        e2.setStyle("display", "none");
      } else {
        e1.setStyle("display", "none");
        e2.setStyle("display", "block");
      }
    };
    window.addEvent("domready", function() {
      if ($("ctrl_accommodations_startDay")) {
        enableStartDay();
        $("ctrl_accommodations_format").addEvent("change", enableStartDay);
      }
    });
  </script>';
	}

	/**
	 * Return all accommodations templates as array
	 *
	 * @return array
	 */
	public function getAccommodationsTemplates()
	{
		return $this->getTemplateGroup('accommodations_');
	}

	/**
	 * Return the sorting options
	 *
	 * @param Contao\DataContainer $dc
	 *
	 * @return array
	 */
	public function getSortingOptions(Contao\DataContainer $dc)
	{
		return array('order_sorting_asc', 'order_sorting_desc', 'order_headline_asc', 'order_headline_desc', 'order_random');
	}
}


/*
 * Offers Part
*/
$GLOBALS['TL_DCA']['tl_module']['palettes']['offerslist']    = '{title_legend},name,headline,type;{config_legend},offers_sets,offers_readerModule,numberOfItems,offers_featured,offers_order,skipFirst,perPage;{template_legend:hide},offers_template,customTpl;{image_legend:hide},imgSize;{protected_legend:hide},protected;{expert_legend:hide},guests,cssID';
$GLOBALS['TL_DCA']['tl_module']['palettes']['offersreader']  = '{title_legend},name,headline,type;{config_legend},offers_sets;{template_legend:hide},offers_template,customTpl;{image_legend:hide},imgSize;{protected_legend:hide},protected;{expert_legend:hide},guests,cssID';

// Add fields to tl_module
$GLOBALS['TL_DCA']['tl_module']['fields']['offers_sets'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['offers_sets'],
	'exclude'                 => true,
	'inputType'               => 'checkbox',
	'options_callback'        => array('tl_module_offers', 'getOffersSet'),
	'eval'                    => array('multiple'=>true, 'mandatory'=>true),
	'sql'                     => "blob NULL"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['offers_featured'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['offers_featured'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options'                 => array('all_items', 'featured', 'unfeatured'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('tl_class'=>'w50 clr'),
	'sql'                     => "varchar(16) NOT NULL default 'all_items'"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['offers_jumpToCurrent'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['offers_jumpToCurrent'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options'                 => array('hide_module', 'show_current', 'all_items'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('tl_class'=>'w50'),
	'sql'                     => "varchar(16) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['offers_readerModule'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['offers_readerModule'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options_callback'        => array('tl_module_offers', 'getReaderModules'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('includeBlankOption'=>true, 'tl_class'=>'w50'),
	'sql'                     => "int(10) unsigned NOT NULL default 0"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['offers_template'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['offers_template'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options_callback'        => array('tl_module_offers', 'getOffersTemplates'),
	'eval'                    => array('tl_class'=>'w50'),
	'sql'                     => "varchar(64) NOT NULL default 'offers_latest'"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['offers_format'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['offers_format'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options'                 => array('offers_day', 'offers_month', 'offers_year'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('tl_class'=>'w50 clr', 'addWizardClass'=>false),
	'wizard' => array
	(
		array('tl_module_offers', 'hideStartDay')
	),
	'sql'                     => "varchar(32) NOT NULL default 'offers_month'"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['offers_startDay'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['offers_startDay'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options'                 => array(0, 1, 2, 3, 4, 5, 6),
	'reference'               => &$GLOBALS['TL_LANG']['DAYS'],
	'eval'                    => array('tl_class'=>'w50'),
	'sql'                     => "smallint(5) unsigned NOT NULL default 0"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['offers_order'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['offers_order'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options_callback'        => array('tl_module_offers', 'getSortingOptions'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('tl_class'=>'w50'),
	'sql'                     => "varchar(32) NOT NULL default 'order_date_asc'"
);


/**
 * Provide miscellaneous methods that are used by the data configuration array.
 */
class tl_module_offers extends Contao\Backend
{

	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import('Contao\BackendUser', 'User');
	}

	/**
	 * Get all offers sets and return them as array
	 *
	 * @return array
	 */
	public function getOffersSet()
	{
		if (!$this->User->isAdmin && !\is_array($this->User->offers))
		{
			return array();
		}

		$arrSets = array();
		$objSets = $this->Database->execute("SELECT id, title FROM tl_offers_set ORDER BY title");

		while ($objSets->next())
		{
			if ($this->User->hasAccess($objSets->id, 'offers'))
			{
				$arrSets[$objSets->id] = $objSets->title;
			}
		}

		return $arrSets;
	}

	/**
	 * Get all offers reader modules and return them as array
	 *
	 * @return array
	 */
	public function getReaderModules()
	{
		$arrModules = array();
		$objModules = $this->Database->execute("SELECT m.id, m.name, t.name AS theme FROM tl_module m LEFT JOIN tl_theme t ON m.pid=t.id WHERE m.type='offersreader' ORDER BY t.name, m.name");

		while ($objModules->next())
		{
			$arrModules[$objModules->theme][$objModules->id] = $objModules->name . ' (ID ' . $objModules->id . ')';
		}

		return $arrModules;
	}

	/**
	 * Hide the start day drop-down if not applicable
	 *
	 * @return string
	 */
	public function hideStartDay()
	{
		return '
  <script>
    var enableStartDay = function() {
      var e1 = $("ctrl_offers_startDay").getParent("div");
      var e2 = $("ctrl_offers_order").getParent("div");
      if ($("ctrl_offers_format").value == "offers_day") {
        e1.setStyle("display", "block");
        e2.setStyle("display", "none");
      } else {
        e1.setStyle("display", "none");
        e2.setStyle("display", "block");
      }
    };
    window.addEvent("domready", function() {
      if ($("ctrl_offers_startDay")) {
        enableStartDay();
        $("ctrl_offers_format").addEvent("change", enableStartDay);
      }
    });
  </script>';
	}

	/**
	 * Return all offers templates as array
	 *
	 * @return array
	 */
	public function getOffersTemplates()
	{
		return $this->getTemplateGroup('offers_');
	}

	/**
	 * Return the sorting options
	 *
	 * @param Contao\DataContainer $dc
	 *
	 * @return array
	 */
	public function getSortingOptions(Contao\DataContainer $dc)
	{
		return array('order_date_asc', 'order_date_desc', 'order_headline_asc', 'order_headline_desc', 'order_random');
	}
}





/*
 * Prices Part
*/
$GLOBALS['TL_DCA']['tl_module']['palettes']['priceslist']    = '{title_legend},name,headline,type;{config_legend},prices_sets,prices_keys,numberOfItems,prices_featured,prices_order,skipFirst,perPage;{template_legend:hide},prices_template,customTpl;{protected_legend:hide},protected;{expert_legend:hide},guests,cssID';

// Add fields to tl_module
$GLOBALS['TL_DCA']['tl_module']['fields']['prices_sets'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['prices_sets'],
	'exclude'                 => true,
	'inputType'               => 'checkbox',
	'options_callback'        => array('tl_module_prices', 'getPricesSets'),
	'eval'                    => array('multiple'=>true, 'mandatory'=>true, 'tl_class'=>'clr'),
	'sql'                     => "blob NULL"
);
$GLOBALS['TL_DCA']['tl_module']['fields']['prices_keys'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['prices_keys'],
	'exclude'                 => true,
	'inputType'               => 'checkbox',
	'options_callback'        => array('tl_module_prices', 'getPricesKeys'),
	'eval'                    => array('multiple'=>true, 'mandatory'=>false, 'tl_class'=>'clr'),
	'sql'                     => "blob NULL"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['prices_featured'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['prices_featured'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options'                 => array('all_items', 'featured', 'unfeatured'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('tl_class'=>'w50 clr'),
	'sql'                     => "varchar(16) NOT NULL default 'all_items'"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['prices_jumpToCurrent'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['prices_jumpToCurrent'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options'                 => array('hide_module', 'show_current', 'all_items'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('tl_class'=>'w50'),
	'sql'                     => "varchar(16) NOT NULL default ''"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['prices_template'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['prices_template'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options_callback'        => array('tl_module_prices', 'getPricesTemplates'),
	'eval'                    => array('tl_class'=>'w50'),
	'sql'                     => "varchar(64) NOT NULL default 'prices_latest'"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['prices_format'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['prices_format'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options'                 => array('prices_day', 'prices_month', 'prices_year'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('tl_class'=>'w50 clr', 'addWizardClass'=>false),
	'wizard' => array
	(
		array('tl_module_prices', 'hideStartDay')
	),
	'sql'                     => "varchar(32) NOT NULL default 'prices_month'"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['prices_startDay'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['prices_startDay'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options'                 => array(0, 1, 2, 3, 4, 5, 6),
	'reference'               => &$GLOBALS['TL_LANG']['DAYS'],
	'eval'                    => array('tl_class'=>'w50'),
	'sql'                     => "smallint(5) unsigned NOT NULL default 0"
);

$GLOBALS['TL_DCA']['tl_module']['fields']['prices_order'] = array
(
	'label'                   => &$GLOBALS['TL_LANG']['tl_module']['prices_order'],
	'exclude'                 => true,
	'inputType'               => 'select',
	'options_callback'        => array('tl_module_prices', 'getSortingOptions'),
	'reference'               => &$GLOBALS['TL_LANG']['tl_module'],
	'eval'                    => array('tl_class'=>'w50'),
	'sql'                     => "varchar(32) NOT NULL default 'order_sorting_asc'"
);


/**
 * Provide miscellaneous methods that are used by the data configuration array.
 */
class tl_module_prices extends Contao\Backend
{

	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import('Contao\BackendUser', 'User');
	}

	/**
	 * Get all prices sets and return them as array
	 *
	 * @return array
	 */
	public function getPricesSets()
	{
		if (!$this->User->isAdmin && !\is_array($this->User->prices))
		{
			return array();
		}

		$arrSets = array();
		$objSets = $this->Database->execute("SELECT id, title FROM tl_prices_set ORDER BY title");

		while ($objSets->next())
		{
			if ($this->User->hasAccess($objSets->id, 'prices'))
			{
				$arrSets[$objSets->id] = $objSets->title;
			}
		}

		return $arrSets;
	}

	/**
	 * Get all prices sets and return them as array
	 *
	 * @return array
	 */
	public function getPricesKeys($dc)
	{
		
		if (!$this->User->isAdmin && !\is_array($this->User->prices))
		{
			return array();
		}

		if($dc->activeRecord->prices_sets == '') {
			return array();
		}

		$sets = [];
		if(!empty($dc->activeRecord->prices_sets)) {
			$pricesSets = unserialize($dc->activeRecord->prices_sets);
			foreach ($pricesSets as $key => $set) {
				array_push($sets, $set);
			}
		}

		// array to string with comma
		$sets = implode(',', $sets);

		$arrSets = array();
		// $objSets = $this->Database->execute("SELECT id, title FROM tl_prices_set ORDER BY title");
		$objSets = $this->Database->prepare("SELECT id,config,title FROM tl_prices_set WHERE id=?")
									 ->execute($sets);
		// $objSets = PricesSetModel::findById($sets);

		

		while ($objSets->next())
		{
			$arrSets[$objSets->title] = [];
			if ($this->User->hasAccess($objSets->id, 'prices'))
			{
				$config = unserialize($objSets->config);
				foreach ($config as $key => $value) {
					$arrSets[$objSets->title][$objSets->id.'__'.$value['key']] = $value['headline'];
				}
			}
		}

		return $arrSets;
	}


	/**
	 * Return all prices templates as array
	 *
	 * @return array
	 */
	public function getPricesTemplates()
	{
		return $this->getTemplateGroup('prices_');
	}

	/**
	 * Return the sorting options
	 *
	 * @param Contao\DataContainer $dc
	 *
	 * @return array
	 */
	public function getSortingOptions(Contao\DataContainer $dc)
	{
		return array('order_sorting_asc', 'order_sorting_desc', 'order_headline_asc', 'order_headline_desc', 'order_random');
	}
}

